@@ -20,8 +20,8 @@
 #include "jamiaccount.h"
 #include "fileutils.h"
 
-#ifdef ENABLE_PLUGIN
 #include "manager.h"
+#ifdef ENABLE_PLUGIN
 #include "plugin/jamipluginmanager.h"
 #endif
 
@@ -32,14 +32,18 @@
 
 namespace jami {
 
-ContactList::ContactList(const std::shared_ptr<crypto::Certificate>& cert,
-                         const std::string& path,
+ContactList::ContactList(const std::string& accountId,
+                         const std::shared_ptr<crypto::Certificate>& cert,
+                         const std::filesystem::path& path,
                          OnChangeCallback cb)
     : path_(path)
     , callbacks_(std::move(cb))
+    , accountId_(accountId)
 {
-    if (cert)
+    if (cert) {
+        trust_ = std::make_unique<dhtnet::tls::TrustStore>(jami::Manager::instance().certStore(accountId_));
         accountTrust_.add(*cert);
+    }
 }
 
 ContactList::~ContactList() {}
@@ -62,13 +66,21 @@ ContactList::save()
 
 bool
 ContactList::setCertificateStatus(const std::string& cert_id,
-                                  const tls::TrustStore::PermissionStatus status)
+                                  const dhtnet::tls::TrustStore::PermissionStatus status)
 {
     if (contacts_.find(dht::InfoHash(cert_id)) != contacts_.end()) {
         JAMI_DBG("Can't set certificate status for existing contacts %s", cert_id.c_str());
         return false;
     }
-    return trust_.setCertificateStatus(cert_id, status);
+    return trust_->setCertificateStatus(cert_id, status);
+}
+
+bool
+ContactList::setCertificateStatus(const std::shared_ptr<crypto::Certificate>& cert,
+                            dhtnet::tls::TrustStore::PermissionStatus status,
+                            bool local)
+{
+    return trust_->setCertificateStatus(cert, status, local);
 }
 
 bool
@@ -88,7 +100,7 @@ ContactList::addContact(const dht::InfoHash& h, bool confirmed, const std::strin
     c->second.conversationId = conversationId;
     c->second.confirmed |= confirmed;
     auto hStr = h.toString();
-    trust_.setCertificateStatus(hStr, tls::TrustStore::PermissionStatus::ALLOWED);
+    trust_->setCertificateStatus(hStr, dhtnet::tls::TrustStore::PermissionStatus::ALLOWED);
     saveContacts();
     callbacks_.contactAdded(hStr, c->second.confirmed);
     return true;
@@ -111,27 +123,22 @@ ContactList::removeContact(const dht::InfoHash& h, bool ban)
     auto c = contacts_.find(h);
     if (c == contacts_.end())
         c = contacts_.emplace(h, Contact {}).first;
-    else if (not c->second.isActive() and c->second.banned == ban)
-        return false;
     c->second.removed = std::time(nullptr);
     c->second.confirmed = false;
     c->second.banned = ban;
     auto uri = h.toString();
-    trust_.setCertificateStatus(uri,
-                                ban ? tls::TrustStore::PermissionStatus::BANNED
-                                    : tls::TrustStore::PermissionStatus::UNDEFINED);
+    trust_->setCertificateStatus(uri,
+                                ban ? dhtnet::tls::TrustStore::PermissionStatus::BANNED
+                                    : dhtnet::tls::TrustStore::PermissionStatus::UNDEFINED);
     if (trustRequests_.erase(h) > 0)
         saveTrustRequests();
     saveContacts();
 #ifdef ENABLE_PLUGIN
-    std::size_t found = path_.find_last_of(DIR_SEPARATOR_CH);
-    if (found != std::string::npos) {
-        auto filename = path_.substr(found + 1);
-        jami::Manager::instance()
-            .getJamiPluginManager()
-            .getChatServicesManager()
-            .cleanChatSubjects(filename, uri);
-    }
+    auto filename = path_.filename().string();
+    jami::Manager::instance()
+        .getJamiPluginManager()
+        .getChatServicesManager()
+        .cleanChatSubjects(filename, uri);
 #endif
     callbacks_.contactRemoved(uri, ban);
     return true;
@@ -202,12 +209,12 @@ ContactList::updateContact(const dht::InfoHash& id, const Contact& contact)
         if (trustRequests_.erase(id) > 0)
             saveTrustRequests();
         if (c->second.isActive()) {
-            trust_.setCertificateStatus(id.toString(), tls::TrustStore::PermissionStatus::ALLOWED);
+            trust_->setCertificateStatus(id.toString(), dhtnet::tls::TrustStore::PermissionStatus::ALLOWED);
             callbacks_.contactAdded(id.toString(), c->second.confirmed);
         } else {
             if (c->second.banned)
-                trust_.setCertificateStatus(id.toString(),
-                                            tls::TrustStore::PermissionStatus::BANNED);
+                trust_->setCertificateStatus(id.toString(),
+                                            dhtnet::tls::TrustStore::PermissionStatus::BANNED);
             callbacks_.contactRemoved(id.toString(), c->second.banned);
         }
     }
@@ -235,14 +242,14 @@ ContactList::loadContacts()
 void
 ContactList::saveContacts() const
 {
-    std::ofstream file(path_ + DIR_SEPARATOR_STR "contacts", std::ios::trunc | std::ios::binary);
+    std::ofstream file(path_ / "contacts", std::ios::trunc | std::ios::binary);
     msgpack::pack(file, contacts_);
 }
 
 void
 ContactList::saveTrustRequests() const
 {
-    std::ofstream file(path_ + DIR_SEPARATOR_STR "incomingTrustRequests",
+    std::ofstream file(path_ / "incomingTrustRequests",
                        std::ios::trunc | std::ios::binary);
     msgpack::pack(file, trustRequests_);
 }
@@ -250,7 +257,7 @@ ContactList::saveTrustRequests() const
 void
 ContactList::loadTrustRequests()
 {
-    if (!fileutils::isFile(fileutils::getFullPath(path_, "incomingTrustRequests")))
+    if (!std::filesystem::is_regular_file(fileutils::getFullPath(path_, "incomingTrustRequests")))
         return;
     std::map<dht::InfoHash, TrustRequest> requests;
     try {
@@ -375,7 +382,6 @@ ContactList::acceptTrustRequest(const dht::InfoHash& from)
     auto i = trustRequests_.find(from);
     if (i == trustRequests_.end())
         return false;
-
     addContact(from, true, i->second.conversationId);
     // Clear trust request
     trustRequests_.erase(i);
@@ -384,10 +390,10 @@ ContactList::acceptTrustRequest(const dht::InfoHash& from)
 }
 
 void
-ContactList::acceptConversation(const std::string& convId)
+ContactList::acceptConversation(const std::string& convId, const std::string& deviceId)
 {
     if (callbacks_.acceptConversation)
-        callbacks_.acceptConversation(convId);
+        callbacks_.acceptConversation(convId, deviceId);
 }
 
 bool
@@ -412,10 +418,7 @@ ContactList::loadKnownDevices()
         std::map<dht::PkId, std::pair<std::string, uint64_t>> knownDevices;
         oh.get().convert(knownDevices);
         for (const auto& d : knownDevices) {
-            /*JAMI_DBG("[Contacts] loading known account device %s %s",
-                    d.second.first.c_str(),
-                    d.first.toString().c_str());*/
-            if (auto crt = tls::CertificateStore::instance().getCertificate(d.first.toString())) {
+            if (auto crt = jami::Manager::instance().certStore(accountId_).getCertificate(d.first.toString())) {
                 if (not foundAccountDevice(crt, d.second.first, clock::from_time_t(d.second.second)))
                     JAMI_WARN("[Contacts] can't add device %s", d.first.toString().c_str());
             } else {
@@ -431,8 +434,7 @@ ContactList::loadKnownDevices()
             std::map<dht::InfoHash, std::pair<std::string, uint64_t>> knownDevices;
             oh.get().convert(knownDevices);
             for (const auto& d : knownDevices) {
-                if (auto crt = tls::CertificateStore::instance().getCertificate(
-                        d.first.toString())) {
+                if (auto crt = jami::Manager::instance().certStore(accountId_).getCertificate(d.first.toString())) {
                     if (not foundAccountDevice(crt,
                                                d.second.first,
                                                clock::from_time_t(d.second.second)))
@@ -449,7 +451,7 @@ ContactList::loadKnownDevices()
 void
 ContactList::saveKnownDevices() const
 {
-    std::ofstream file(path_ + DIR_SEPARATOR_STR "knownDevices", std::ios::trunc | std::ios::binary);
+    std::ofstream file(path_ / "knownDevices", std::ios::trunc | std::ios::binary);
 
     std::map<dht::PkId, std::pair<std::string, uint64_t>> devices;
     for (const auto& id : knownDevices_)
@@ -506,12 +508,12 @@ ContactList::foundAccountDevice(const std::shared_ptr<dht::crypto::Certificate>&
     auto it = knownDevices_.emplace(id, KnownDevice {crt, name, updated});
     if (it.second) {
         JAMI_DBG("[Contacts] Found account device: %s %s", name.c_str(), id.toString().c_str());
-        tls::CertificateStore::instance().pinCertificate(crt);
+        jami::Manager::instance().certStore(accountId_).pinCertificate(crt);
         if (crt->ocspResponse) {
             unsigned int status = crt->ocspResponse->getCertificateStatus();
             if (status == GNUTLS_OCSP_CERT_REVOKED) {
                 JAMI_ERR("Certificate %s has revoked OCSP status", id.to_c_str());
-                trust_.setCertificateStatus(crt, tls::TrustStore::PermissionStatus::BANNED, false);
+                trust_->setCertificateStatus(crt, dhtnet::tls::TrustStore::PermissionStatus::BANNED, false);
             }
         }
         saveKnownDevices();
